-- Copyright © 2012 Xianghar  <xian@zron.de>
-- All Rights Reserved.

if select(2,UnitClass("player")) ~= "MONK" then return end

local _, MonkTimers = ...

local L = LibStub("AceLocale-3.0"):GetLocale("MonkTimers_GUI", true)

local libmov = LibStub:GetLibrary("LibMovable-1.0")

local SpellIDs = MonkTimers.SpellIDs
local SpellTextures = MonkTimers.SpellTextures
local SpellNames = MonkTimers.SpellNames
local AvailableSpells = MonkTimers.AvailableSpells
local stagger, staggerButton

local StaggerEvent, ChiBarEvent

function  MonkTimers.CreateStaggerAndHarmonyBars()
	stagger = CreateFrame("StatusBar", "MonkTimers_StaggerBar", UIParent, "XiTimersTimerBarTemplate, SecureActionButtonTemplate, SecureHandlerBaseTemplate")
	stagger:ClearAllPoints()
	stagger:SetPoint("CENTER", UIParent, "CENTER")
	libmov.RegisterMovable("XiTimers", stagger, XiTimers.getAnchorTable, L["Stagger Bar"])
	staggerButton = CreateFrame("Button", "MonkTimers_StaggerBarButton", MonkTimers_StaggerBar, "ActionButtonTemplate, SecureActionButtonTemplate")
	RegisterStateDriver(stagger, "petbattle", "[petbattle][overridebar][possessbar] active; none")
	RegisterStateDriver(stagger, "combat", "[combat] active; none")
	stagger:WrapScript(stagger, "OnAttributeChanged", [[
		if not self:GetAttribute("active") then return end
		if name == "state-petbattle" then
			if value == "active" then
				self:Hide()
			elseif not self:GetAttribute("HideOOC") then
				self:Show()
			end
		elseif name == "state-combat" then
			if value == "active" then
				self:Show()
			elseif self:GetAttribute("HideOOC") then
				self:Hide()
			end
		end
	]])
	RegisterStateDriver(MonkTimersHarmonyBar, "petbattle", "[petbattle][overridebar][possessbar] active; none")
	RegisterStateDriver(MonkTimersHarmonyBar, "combat", "[combat] active; none")
	stagger:WrapScript(MonkTimersHarmonyBar, "OnAttributeChanged", [[
		if not self:GetAttribute("active") then return end
		if name == "state-petbattle" then
			if value == "active" then
				self:Hide()
			elseif not self:GetAttribute("HideOOC") then
				self:Show()
			end
		elseif name == "state-combat" then
			if value == "active" then
				self:Show()
			elseif self:GetAttribute("HideOOC") then
				self:Hide()
			end
		end
	]])

	stagger.icon = getglobal("MonkTimers_StaggerBarIcon")
    stagger.background = getglobal("MonkTimers_StaggerBarBackground")
    stagger.icon:SetTexture(SpellTextures[SpellIDs.StaggerLight])
    stagger.icon:Show()
    stagger.icon:SetPoint("RIGHT", stagger, "LEFT")
    stagger.background:Show()
    stagger.background:SetValue(1)
    stagger.background:SetWidth(120)
    stagger.background:SetStatusBarColor(0, 1, 0, 0.3)
    stagger:SetWidth(120)
    stagger:SetScript("OnEvent", StaggerEvent)	
    stagger.text = getglobal("MonkTimers_StaggerBarTime")
	stagger.text:ClearAllPoints()
	stagger.text:SetPoint("CENTER", stagger, "CENTER")
	MonkTimers.stagger = stagger
    MonkTimers.staggerButton = staggerButton
    --staggerButton:SetWidth(100)
	--staggerButton:Hide()
    --staggerButton:SetHeight(stagger.background:GetHeight())
	staggerButton:ClearAllPoints()
	staggerButton:SetAllPoints(stagger)
    staggerButton:SetPoint("CENTER", stagger, "CENTER")
    staggerButton:SetNormalTexture(nil)
    staggerButton:SetHighlightTexture("Interface\\AddOns\\TotemTimers\\MaelstromHilight")
    staggerButton:SetPushedTexture("Interface\\AddOns\\TotemTimers\\MaelstromPushed")
    staggerButton:SetAttribute("*type*", "spell")
    staggerButton:RegisterForClicks("LeftButtonUp", "RightButtonUp")
    staggerButton:SetAttribute("*spell1", SpellNames[SpellIDs.PurifyingBrew])
	
	MonkTimers.harmonyBar = MonkTimersHarmonyBar
	
	MonkTimersHarmonyBar:SetScale(0.9)
	MonkTimersHarmonyBar:SetPoint("CENTER", UIParent, "CENTER")
	libmov.RegisterMovable("XiTimers", MonkTimersHarmonyBar, XiTimers.getAnchorTable, L["Chi Bar"])
	MonkTimersHarmonyBarEnergyBar:SetFrameLevel(MonkTimersHarmonyBarEnergyBar:GetFrameLevel()-1)
	MonkTimersHarmonyBar:SetScript("OnEvent", ChiBarEvent)
	MonkTimersHarmonyBar.energyBar = MonkTimersHarmonyBarEnergyBar
	MonkTimersHarmonyBar.maxLight = 1
	MonkTimersHarmonyBar:SetScript("OnUpdate", ChiBarUpdate)
	
end



local staggerLight = SpellNames[SpellIDs.StaggerLight]
local staggerModerate = SpellNames[SpellIDs.StaggerModerate]
local staggerHeavy = SpellNames[SpellIDs.StaggerHeavy]
local lightTexture = SpellTextures[SpellIDs.StaggerLight]
local moderateTexture = SpellTextures[SpellIDs.StaggerModerate]
local heavyTexture = SpellTextures[SpellIDs.StaggerHeavy]

local format = string.format
local UnitHealthMax = UnitHealthMax

local function SetStaggerText(amount, expiration)
	local maxHealth = UnitHealthMax("player")
	local percent = amount/maxHealth*100
	local total = amount*(expiration-GetTime())
	--if value > 1000000 then
	--	stagger.text:SetText(format("%.2fm", value/1000000) .. " ("..format("%.1f", percent).."%)")
	local text = ""
	local textformat = "%u"
	if amount > 1000 then
		textformat = "%uk"
		amount = amount/1000
		total = total/1000
	end
	if MonkTimers.ActiveProfile.StaggerDot then
		text = format(textformat, amount)
		if MonkTimers.ActiveProfile.StaggerDotTotal then
			text = text.."/"
		end
	end
	if MonkTimers.ActiveProfile.StaggerDotTotal then
		text = text .. format(textformat, total)
	end
	if MonkTimers.ActiveProfile.StaggerDotPercent then
		text = text .. format(" (%.1f%%)", percent)
	end
	stagger.text:SetText(text)
end


local function SetStagger(texture, expiration, amount)
	if texture == lightTexture then
		stagger:SetStatusBarColor(0,1,0)
		stagger.background:SetStatusBarColor(0,1,0,0.3)
		if staggerButton.overlay then
			--staggerButton.overlay.spark:SetVertexColor(0,1,0,0.5)
			--staggerButton.overlay.innerGlow:SetVertexColor(0,1,0,0.5)
			--staggerButton.overlay.innerGlowOver:SetVertexColor(0,1,0,0.5)
			staggerButton.overlay.outerGlow:SetVertexColor(0,1,0,0.3)
			--staggerButton.overlay.outerGlowOver:SetVertexColor(0,1,0,0.5)
			--staggerButton.overlay.ants:SetVertexColor(0,1,0,0.5)
		end
	elseif texture == moderateTexture then
		stagger:SetStatusBarColor(1,1,0)
		stagger.background:SetStatusBarColor(1,1,0,0.3)
		if staggerButton.overlay then
			staggerButton.overlay.outerGlow:SetVertexColor(1,1,0,0.3)
		end
	else
		stagger:SetStatusBarColor(1,0,0)
		stagger.background:SetStatusBarColor(1,0,0,0.3)
		if staggerButton.overlay then
			staggerButton.overlay.outerGlow:SetVertexColor(1,0,0,0.3)
		end
	end
	local maxhealth = UnitHealthMax("player")*0.06
	stagger:SetMinMaxValues(0,maxhealth)
	stagger:SetValue(amount)
	stagger.icon:SetTexture(texture)
	--stagger.background:SetMinMaxValues(0,1)
	--stagger.background:SetValue(1)
	SetStaggerText(amount,expiration)	
end


local function MT_ShowOverlayGlow(self)
	if ( self.overlay ) then
		if ( self.overlay.animOut:IsPlaying() ) then
			self.overlay.animOut:Stop();
			self.overlay.animIn:Play();
		end
	else
		self.overlay = CreateFrame("Frame", "MonkTimers_StaggerOverlay", UIParent, "ActionBarButtonSpellActivationAlert")
		local frameWidth, frameHeight = self:GetSize()
		self.overlay:SetParent(self)
		self.overlay:ClearAllPoints()
		--Make the height/width available before the next frame:
		self.overlay:SetSize(frameWidth * 1.4, frameHeight * 1.4)
		self.overlay:SetPoint("TOPLEFT", self, "TOPLEFT", -frameWidth * 0.2, frameHeight * 0.2)
		self.overlay:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", frameWidth * 0.2, -frameHeight * 0.2)
		self.overlay.animOut:SetScript("OnFinished", function(self) self:GetParent():Hide() end)
		self.overlay:SetScript("OnHide",
			function(self)
				if self.animOut:IsPlaying() then
					self.animOut:Stop()				
				end
			end
		)
		self.overlay.animIn:Play()
	end
end

local function MT_HideOverlayGlow(self)
	if ( self.overlay ) then
		if ( self.overlay.animIn:IsPlaying() ) then
			self.overlay.animIn:Stop();
		end
		if ( self:IsVisible() ) then
			self.overlay.animOut:Play();
		else
			self.overlay:Hide()
		end
	end
end

local purifier = SpellNames[SpellIDs.Purifier]
local staggerGlowActive = false

StaggerEvent = function(self, event)
	if event == "UNIT_AURA" then
		-- brewmaster t15 4pc
		local _,_,_,_,_,duration,expiration = UnitBuff("player", purifier)
		if duration and duration > 0 then
			if not staggerGlowActive then
				staggerGlowActive = true
				MT_ShowOverlayGlow(staggerButton)	
			end
		elseif staggerGlowActive then
			staggerGlowActive = false
			MT_HideOverlayGlow(staggerButton)	
		end
		
		local _,_,_,_,_,_,expiration,_,_,_,_,_,_,_,_,_,_,amount = UnitDebuff("player", staggerLight)
		--print(expiration, amount)
		if amount then SetStagger(lightTexture, expiration, amount) return end
		local _,_,_,_,_,_,expiration,_,_,_,_,_,_,_,_,_,_, amount = UnitDebuff("player", staggerModerate)
		if amount then SetStagger(moderateTexture, expiration, amount) return end
		local _,_,_,_,_,_,expiration,_,_,_,_,_,_,_,_,_,_, amount = UnitDebuff("player", staggerHeavy)
		if amount then SetStagger(heavyTexture, expiration, amount) return end
		stagger.icon:SetTexture(texture)
		stagger:SetValue(0)
		stagger.text:SetText("")
	elseif event == "PLAYER_REGEN_ENABLED" then
       --[[ if MonkTimers.ActiveProfile.HideTimersOOC then
            self:Hide()
        end]]
        self:SetAlpha(MonkTimers.ActiveProfile.TimerOOCAlpha)
    elseif event == "PLAYER_REGEN_DISABLED" then
       --[[ if not self:IsVisible() then
            self:Show()
        end]]
		self:SetAlpha(1)
    end
end




local function SetEnergy(lightEnergy, active)
	if ( active ) then
		if (lightEnergy.deactivate:IsPlaying()) then
			lightEnergy.deactivate:Stop();
		end
		
		if (not lightEnergy.active and not lightEnergy.activate:IsPlaying()) then
			lightEnergy.activate:Play();
			lightEnergy.active = true;
		end
	else
		if (lightEnergy.activate:IsPlaying()) then
			lightEnergy.activate:Stop();
		end
		
		if (lightEnergy.active and not lightEnergy.deactivate:IsPlaying()) then
			lightEnergy.deactivate:Play();
			lightEnergy.active = false;
		end
	end
end



ChiBarEvent = function(self, event, unit, power)
	if event == "UNIT_POWER_FREQUENT" then
		if power == "CHI" or power == "DARK_FORCE" then
			local light = UnitPower("player", SPELL_POWER_CHI)
			for i = 1, self.maxLight do
				SetEnergy(self.LightEnergy[i], i<=light);
			end
		end
	elseif event == "UNIT_MAXPOWER" then
		local maxLight = UnitPowerMax("player", SPELL_POWER_CHI );
		if ( self.maxLight ~= maxLight ) then
			local startX, xOffset, orbOff, lightOrb;

			if ( maxLight == 4 ) then
				startX = -43;
				xOffset = 5;
				orbOff = "MonkUI-OrbOff";
				lightOrb = "MonkUI-LightOrb";
			elseif (maxLight == 5 ) then
				startX = -46;
				xOffset = 1;
				orbOff = "MonkUI-OrbOff";
				lightOrb = "MonkUI-LightOrb";
			else
				startX = -54;
				xOffset = 0;
				orbOff = "MonkUI-OrbOff-small";
				lightOrb = "MonkUI-LightOrb-small";
			end

			for i = 1,maxLight do
				local orb = self.LightEnergy[i];
				if (not orb) then
					orb = CreateFrame("Frame", nil, MonkTimersHarmonyBar, "MonkLightEnergyTemplate");
				end
				orb:ClearAllPoints();
				orb.Glow:SetAtlas(lightOrb, true);
				orb.OrbOff:SetAtlas(orbOff, true);
				if (i == 1) then
					orb:SetPoint("LEFT", startX, 1);
				else
					local prev = self.LightEnergy[i - 1];
					orb:SetPoint("LEFT", prev, "RIGHT", xOffset, 0);
				end
				orb:Show();
			end
			
			for i = maxLight+1, #self.LightEnergy do
				local orb = self.LightEnergy[i];
				if (orb) then
					orb:Hide();
				end
			end

			self.maxLight = maxLight;
		end
	elseif event == "PLAYER_REGEN_ENABLED" then
        if MonkTimers.ActiveProfile.HideTimersOOC then
            self:Hide()
        end
        self:SetAlpha(MonkTimers.ActiveProfile.TimerOOCAlpha)
    elseif event == "PLAYER_REGEN_DISABLED" then
        if not self:IsVisible() then
            self:Show()
        end
		self:SetAlpha(1)
	elseif event == "PET_BATTLE_OPENING_START" then
		self:Hide()
	elseif event == "PET_BATTLE_CLOSE" then
		if self.active and not MonkTimers.ActiveProfile.HideTimersOOC then
			self:Show()
		end
    end
end
MonkTimers.ChiBarEvent = ChiBarEvent

ChiBarUpdate = function(self)
	local power = UnitPower("player", self.powerType)
	self.energyBar:SetValue(power)
	if power > 1000 then
		power = format("%.1fk", power/1000)
	end
	self.energyBar.text:SetText(power)
end


--[[
local introIcon = CreateFrame("FRAME", "MonkTimers_TimerIntro", UIParent, "IconIntroTemplate")

function MonkTimers.TimerIntroAnimation(spell)
	local doAnimation = false
	for i=1,#cds do
		if cds[i].button.cdspell == spell then doAnimation = i end
	end
	if not doAnimation then return end
	introIcon:ClearAllPoints()
	introIcon:SetPoint("CENTER", cds[doAnimation].button, 0,0)
	introIcon.icon.icon:SetTexture(SpellTextures[spell])
	introIcon.icon.slot = 0
	introIcon.icon.flyin:Play(1)
end
]]
